"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const types_1 = require("@babel/types");
// TODO: we should probably provide an Elm package that offers this by default.
// NoRedInk/elm-asset-path comes close but uses an opaque type rather than allowing type aliases.
const defaultPluginOptions = {
    package: "cultureamp/babel-elm-assets-plugin",
    module: "WebpackAsset",
    function: "assetUrl"
};
exports.defaultPluginOptions = defaultPluginOptions;
const plugin = ({}) => {
    /** An append-only list of error descriptions. */
    const errors = [];
    const name = "babel-elm-assets-plugin";
    return {
        name,
        post: () => {
            if (errors.length > 0) {
                // report errors and throw
                throw new Error(`${name}:\n\t` + errors.join("\n\t"));
            }
        },
        visitor: {
            CallExpression: (path, { opts }) => {
                const options = Object.assign({}, defaultPluginOptions, opts);
                if (!isAssetExpression(path.node, options))
                    return;
                const [filePathNode] = path.node.arguments;
                path.replaceWith(types_1.memberExpression(types_1.callExpression(types_1.identifier("require"), path.node.arguments), types_1.identifier("default")));
                if (!types_1.isStringLiteral(filePathNode)) {
                    const name = `${options.module}.${options.function} (from ${options.package})`;
                    errors.push(`When using ${name} you must provide the asset path as a constant string`);
                }
            }
        }
    };
};
/**
 * Returns true if the call expression matches the Asset function signature
 * specified in our plugin config.
 */
const isAssetExpression = (expression, options) => {
    const taggerName = [
        options.package.replace(/-/g, "_").replace(/\//g, "$"),
        options.module.replace(/\./g, "$"),
        options.function
    ].join("$");
    return (types_1.isIdentifier(expression.callee) && expression.callee.name === taggerName);
};
exports.default = plugin;
